@@ -1,7 +1,7 @@
 /**
  * \file timer/timer.c
  * \brief Timer Interface
- * \author Jaroslav Kysela <perex@suse.cz>
+ * \author Jaroslav Kysela <perex@perex.cz>
  * \date 1998-2001
  *
  * Timer Interface is designed to access timers.
@@ -9,7 +9,7 @@
  */
 /*
  *  Timer Interface - main file
- *  Copyright (c) 1998-2001 by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) 1998-2001 by Jaroslav Kysela <perex@perex.cz>
  *
  *
  *   This library is free software; you can redistribute it and/or modify
@@ -67,15 +67,10 @@ This example shows opening a timer device and reading of timer events.
  * \anchor example_test_timer
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <sys/ioctl.h>
 #include "timer_local.h"
 
+#include <signal.h>
+
 static int snd_timer_open_conf(snd_timer_t **timer,
 			       const char *name, snd_config_t *timer_root,
 			       snd_config_t *timer_conf, int mode)
